# -*- coding: utf-8 -*-

import pygame
from pygame.locals import *

import gui
from gui import render_text
from data import get_path

class Scene (object):
	def __init__(self, manager):
		self.manager = manager

	def init(self):
		pass

	def think(self):
		pygame.event.clear()
	
	def render(self, surf):
		pass

class TitleScene (Scene):
	def init(self):
		sprite = pygame.image.load(get_path('grass.png')).convert()
		sprite = pygame.transform.scale(sprite, (64, 64))

		w, h = self.manager.display.get_size()
		surf = pygame.Surface((w, h + 64))
	
		for x in range(w/64):
			for y in range(h/64 + 1):
				surf.blit(sprite, (x*64, y*64))
		self.back = surf
		self.title = render_text('DejaVuSans-Bold.ttf', 128, 'Green', (255, 255, 255))

		self.dispatcher = gui.Dispatcher()
		button = gui.Button((10, 700, 160, 56), 'START', fontsize=48)
		self.dispatcher.add_widget(button)
		button.clicked = self.start

		button = gui.Button((964, 724, 43, 20), 'EXIT')
		button.clicked = self.exit
		self.dispatcher.add_widget(button)
		
		self.shader = pygame.Surface((w, h)).convert_alpha()

		self.offset = 0
		self.done = False
	
	def start(self, *args):
		self.done = 'play'
	
	def exit(self, *args):
		self.done = 'exit'

	def think(self):
		for ev in pygame.event.get():
			self.dispatcher.handle_event(ev)	

		if self.done:
			return self.done, ()

		self.offset = (self.offset + 1) % 64

	def render(self, surf):
		w, h = self.back.get_size()
		surf.blit(self.back, (0, 0), (0, self.offset, w, h))
		surf.blit(self.title, (50, 50))
		self.dispatcher.draw_all(surf)

class ShadeScene(Scene):
	shade = 128

	def __init__(self, manager, prev_scene):
		Scene.__init__(self, manager)
		self.prev_scene = prev_scene
	
	def init(self):
		self.size = self.manager.display.get_size()
		self.buf = pygame.Surface(self.size)
		self.shader = pygame.Surface(self.size).convert_alpha()

		self.counter = 0

		self.prev_scene.render(self.buf)

	def render(self, surf):
		if self.counter < self.shade:
			self.counter += 5
			self.shader.fill((0, 0, 0, self.counter))

		surf.blit(self.buf, (0, 0))
		surf.blit(self.shader, (0, 0))

class PauseScene (ShadeScene):
	def init(self):
		ShadeScene.init(self)
		self.text = render_text(None, 20, 'Press ESC to return to the game.', (255, 255, 255))
		self.dispatcher = gui.Dispatcher()
		b = gui.Button((880, 700, 130, 20), 'Back to game')
		b.clicked = self.back
		self.dispatcher.add_widget(b)
		b = gui.Button((880, 724, 130, 20), 'Back to title')
		b.clicked = self.exit
		self.dispatcher.add_widget(b)

		self.done = False

	def back(self, *args):
		self.done = self.prev_scene

	def exit(self, *args):
		self.done = 'title'

	def think(self):
		for ev in pygame.event.get():
			if ev.type == KEYDOWN and ev.key == K_ESCAPE:
				pygame.event.clear()
				return self.prev_scene, ()
			self.dispatcher.handle_event(ev)
		if self.done:
			return self.done, ()

	
	def render(self, surf):
		ShadeScene.render(self, surf)
		sw, sh = surf.get_size()
		tw, th = self.text.get_size()
		x = (sw - tw) / 2
		y = (sh - th) / 2
		surf.blit(self.text, (x, y))
		self.dispatcher.draw_all(surf)

class EndScene (ShadeScene):
	def init(self):
		ShadeScene.init(self)
		self.text = render_text(None, 20, self.end_text, (255, 255, 255))
		self.dispatcher = gui.Dispatcher()
		button = gui.Button((900, 724, 100, 20), 'CONTINUE')
		button.clicked = self.clicked
		self.dispatcher.add_widget(button)
		self.done = False

	def clicked(self, *args):
		self.done = True

	def think(self):
		for ev in pygame.event.get():
			if ev.type == KEYDOWN and ev.key == K_ESCAPE:
				self.done()
			else:
				self.dispatcher.handle_event(ev)

		if self.done:
			return 'title', ()
	
	def render(self, surf):
		ShadeScene.render(self, surf)

		x = (surf.get_width() - self.text.get_width()) / 2
		y = (surf.get_height() - self.text.get_height()) / 2
		surf.blit(self.text, (x, y))

		self.dispatcher.draw_all(surf)

class WonScene (EndScene):
	end_text = 'Congratulations! You saved the area.'

class BankruptScene (EndScene):
	end_text = 'You went bankrupt and city gradually overtook everything.'

class LynchedScene (EndScene):
	end_text = 'You destroyed the natural landscape and your supporters abandoned you.'

