import pygame
from pygame import *

import gui

from scene import WonScene, BankruptScene, LynchedScene, PauseScene, \
		TitleScene
from play_scene import PlayScene
from sounds import preload_samples, play_sample

class SceneManager (object):
	scenes = {
		'play': PlayScene,
		'won': WonScene,
		'bankrupt': BankruptScene,
		'lynched': LynchedScene,
		'pause': PauseScene,
		'title': TitleScene,
	}

	start_scene = 'title'

	def __init__(self):
		self.scene = None
	
	def init(self):
		pygame.mixer.pre_init(44100)
		pygame.init()
		self.display = pygame.display.set_mode((1024, 768), HWSURFACE | DOUBLEBUF)

		gui.Widget.fg_color = (255, 255, 255)
		gui.Widget.bg_color = (0, 64, 0)
		gui.Widget.hl_color = (0, 128, 0)


	def switch_scene(self, next, args):
		if isinstance(next, basestring):
			self.scene = self.scenes[next](self, *args)
			self.scene.init()
		else:
			self.scene = next

	def run(self):
		clock = pygame.time.Clock()

		self.running = 1
		self.switch_scene(self.start_scene, ())

		while self.running:
			clock.tick(30)
			self.display.fill(0)

			if pygame.event.get(QUIT) or not self.scene:
				break

			next = self.scene.think()
			if next:
				next, args = next
				if next == 'exit':
					self.running = False
				else:
					self.switch_scene(next, args)

			self.scene.render(self.display)
			
			pygame.display.flip()
		
def run():
	"""Run the game.
	"""
	sm = SceneManager()
	sm.init()
	sm.run()


if __name__ == '__main__':
	run()
